<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Converter;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Dedi\SyliusQuotePlugin\Model\OrderInterface;
use Dedi\SyliusQuotePlugin\Model\OrderItemInterface;
use Sylius\Component\Core\Factory\CartItemFactoryInterface;
use Sylius\Component\Order\Modifier\OrderItemQuantityModifierInterface;
use Sylius\Component\Order\Processor\OrderProcessorInterface;

class QuoteToCartConverter implements QuoteToCartConverterInterface
{
    private CartItemFactoryInterface $cartItemFactory;

    private OrderItemQuantityModifierInterface $orderItemQuantityModifier;

    private OrderProcessorInterface $orderProcessor;

    public function __construct(
        CartItemFactoryInterface $cartItemFactory,
        OrderItemQuantityModifierInterface $orderItemQuantityModifier,
        OrderProcessorInterface $orderProcessor
    ) {
        $this->cartItemFactory = $cartItemFactory;
        $this->orderItemQuantityModifier = $orderItemQuantityModifier;
        $this->orderProcessor = $orderProcessor;
    }

    public function convert(QuoteInterface $quote, OrderInterface $cart): void
    {
        $cart->setQuote($quote);

        /** @var QuoteItemInterface $quoteItem */
        foreach ($quote->getItems() as $quoteItem) {
            /** @var OrderItemInterface $cartItem */
            $cartItem = $this->cartItemFactory->createForCart($cart);

            $this->setCartItemPropertiesFromQuote($quoteItem, $cartItem);

            $this->orderItemQuantityModifier->modify($cartItem, $quoteItem->getQuantity());
        }

        $this->orderProcessor->process($cart);
    }

    protected function setCartItemPropertiesFromQuote(QuoteItemInterface $quoteItem, OrderItemInterface $orderItem): void
    {
        $orderItem->setVariant($quoteItem->getVariant());
        $orderItem->setProductName($quoteItem->getProductName());
        $orderItem->setVariantName($quoteItem->getVariantName());
        $orderItem->setQuoteItem($quoteItem);
    }
}
