<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Entity;

use DateTimeInterface;
use Dedi\SyliusQuotePlugin\Enum\QuoteStateEnum;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Sylius\Component\Channel\Model\ChannelInterface;
use Sylius\Component\Core\Model\CustomerInterface;

class Quote implements QuoteInterface
{
    protected ?int $id = null;

    protected ?string $number = null;

    protected ?ChannelInterface $channel = null;

    protected ?CustomerInterface $customer = null;

    protected Collection $items;

    protected ?int $total = null;

    protected ?string $currencyCode = null;

    protected ?string $customerComment = null;

    protected ?string $informationComment = null;

    protected Collection $internalComments;

    protected ?string $state = QuoteStateEnum::STATE_NEW;

    protected ?DateTimeInterface $createdAt = null;

    protected ?DateTimeInterface $updatedAt = null;

    protected ?DateTimeInterface $expirationDate = null;

    protected ?string $countryCode = null;

    protected ?string $localeCode = null;

    public function __construct()
    {
        $this->items = new ArrayCollection();
        $this->internalComments = new ArrayCollection();
        $this->createdAt = new \DateTime();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNumber(): ?string
    {
        return $this->number;
    }

    public function setNumber(?string $number): self
    {
        $this->number = $number;

        return $this;
    }

    public function getChannel(): ?ChannelInterface
    {
        return $this->channel;
    }

    public function setChannel(?ChannelInterface $channel): self
    {
        $this->channel = $channel;

        return $this;
    }

    public function getCustomer(): ?CustomerInterface
    {
        return $this->customer;
    }

    public function setCustomer(?CustomerInterface $customer): self
    {
        $this->customer = $customer;

        return $this;
    }

    public function getItems(): Collection
    {
        return $this->items;
    }

    public function setItems(Collection $items): self
    {
        $this->items = $items;

        return $this;
    }

    public function getTotal(): ?int
    {
        return $this->total;
    }

    public function setTotal(?int $total): self
    {
        $this->total = $total;

        return $this;
    }

    public function getCurrencyCode(): ?string
    {
        return $this->currencyCode;
    }

    public function setCurrencyCode(?string $currencyCode): self
    {
        $this->currencyCode = $currencyCode;

        return $this;
    }

    public function getCustomerComment(): ?string
    {
        return $this->customerComment;
    }

    public function setCustomerComment(?string $customerComment): self
    {
        $this->customerComment = $customerComment;

        return $this;
    }

    public function getInformationComment(): ?string
    {
        return $this->informationComment;
    }

    public function setInformationComment(?string $informationComment): self
    {
        $this->informationComment = $informationComment;

        return $this;
    }

    public function getInternalComments(): Collection
    {
        return $this->internalComments;
    }

    public function setInternalComments(Collection $internalComments): self
    {
        $this->internalComments = $internalComments;

        return $this;
    }

    public function addInternalComment(QuoteInternalComment $comment): void
    {
        if (!$this->hasInternalComment($comment)) {
            $this->internalComments->add($comment);

            $comment->setQuote($this);
        }
    }

    public function hasInternalComment(QuoteInternalComment $quoteComment): bool
    {
        return $this->internalComments->contains($quoteComment);
    }

    public function removeInternalComment(QuoteInternalComment $quoteComment): void
    {
        if ($this->hasInternalComment($quoteComment)) {
            $this->internalComments->removeElement($quoteComment);
        }
    }

    public function getState(): ?string
    {
        return $this->state;
    }

    public function setState(?string $state): self
    {
        $this->state = $state;

        return $this;
    }

    public function getCreatedAt(): ?DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(?DateTimeInterface $createdAt): self
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(?DateTimeInterface $updatedAt): self
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function getExpirationDate(): ?DateTimeInterface
    {
        return $this->expirationDate;
    }

    public function setExpirationDate(?DateTimeInterface $expirationDate): self
    {
        $this->expirationDate = $expirationDate;

        return $this;
    }

    public function getCountryCode(): ?string
    {
        return $this->countryCode;
    }

    public function setCountryCode(?string $countryCode): self
    {
        $this->countryCode = $countryCode;

        return $this;
    }

    public function getLocaleCode(): ?string
    {
        return $this->localeCode;
    }

    public function setLocaleCode(?string $localeCode): self
    {
        $this->localeCode = $localeCode;

        return $this;
    }

    public function addItem(QuoteItemInterface $quoteItem): void
    {
        if ($this->hasItem($quoteItem)) {
            return;
        }

        $this->items->add($quoteItem);
        $quoteItem->setQuote($this);

        $this->recalculateTotal();
    }

    public function hasItem(QuoteItemInterface $quoteItem): bool
    {
        return $this->items->contains($quoteItem);
    }

    public function removeItem(QuoteItemInterface $quoteItem): void
    {
        if ($this->hasItem($quoteItem)) {
            $this->items->removeElement($quoteItem);

            $quoteItem->setQuote(null);

            $this->recalculateTotal();
        }
    }

    public function recalculateTotal(): void
    {
        $this->total = 0;

        /** @var QuoteItemInterface $item */
        foreach ($this->items as $item) {
            $this->total += $item->getTotal();
        }
    }

    public function getItemsTotal(): int
    {
        $total = 0;
        /** @var QuoteItemInterface $item */
        foreach ($this->items as $item) {
            $total += $item->getTotal();
        }

        return $total;
    }

    public function isEmpty(): bool
    {
        return $this->items->isEmpty();
    }
}
